<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper\Catalog;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Api\Data\CategoryInterface;
use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\Eav\Model\Config;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 *
 */
class CategoryHelper
{
    /** @var string */
    public const XML_REFILO_SEARCH_GENERAL_USE_CATEGORY_FULL_PATH = 'refilo_search/general/use_category_full_path';

    /** @var string */
    public const XML_REFILO_SEARCH_GENERAL_CATEGORY_FULL_PATH_SEPARATOR = 'refilo_search/general/category_full_path_separator';

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var array
     */
    private $categoryCache = [];

    /**
     * @var CollectionFactory
     */
    private $categoryCollectionFactory;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var \Magento\Eav\Model\Config
     */
    private $eavConfig;

    /**
     * @var int
     */
    private $isActiveAttributeId;
    /**
     * @var \Magento\Framework\EntityManager\MetadataPool
     */
    private $metadataPool;
    /**
     * @var array
     */
    private $anchorCategories = null;
    /**
     * @var \Magento\Catalog\Api\CategoryRepositoryInterface
     */
    private $categoryRepository;

    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param CollectionFactory $categoryCollectionFactory
     * @param \Magento\Catalog\Api\CategoryRepositoryInterface $categoryRepository
     * @param \Magento\Framework\App\ResourceConnection $resourceConnection
     * @param StoreManagerInterface $storeManager
     * @param \Magento\Eav\Model\Config $eavConfig
     * @param \Magento\Framework\EntityManager\MetadataPool $metadataPool
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        CollectionFactory $categoryCollectionFactory,
        CategoryRepositoryInterface $categoryRepository,
        ResourceConnection $resourceConnection,
        StoreManagerInterface $storeManager,
        Config $eavConfig,
        MetadataPool $metadataPool
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->storeManager = $storeManager;
        $this->resourceConnection = $resourceConnection;
        $this->eavConfig = $eavConfig;
        $this->metadataPool = $metadataPool;
        $this->categoryRepository = $categoryRepository;
    }

    /**
     * Used to show full name in the search result
     * @param CategoryInterface| Category $category
     */
    public function getFullName($category): ?string
    {
        $useFullName = $this->scopeConfig->isSetFlag
        (
            self::XML_REFILO_SEARCH_GENERAL_USE_CATEGORY_FULL_PATH,
            ScopeInterface::SCOPE_STORE,
            $category->getStore()
        );

        if (!$useFullName) {
            return $category->getName();
        }

        $names = [$category->getName()];
        $parent = $category->getParentCategory();

        while (true) {
            if ((int)$parent->getLevel() <= 1 || !$parent->getParentId()) {
                break;
            }

            array_unshift($names, $parent->getName());

            $parent = $parent->getParentCategory();
        }

        $separator = $this->scopeConfig->getValue
        (
            self::XML_REFILO_SEARCH_GENERAL_CATEGORY_FULL_PATH_SEPARATOR,
            ScopeInterface::SCOPE_STORE,
            $category->getStore()
        ) ?? ' / ';
        return implode($separator, $names);
    }

    public function getParentCategory()
    {
    }

    /**
     * @param CategoryInterface| Category | int $category
     * @param $store
     * @return void
     * @throws LocalizedException
     */
    public function getBreadcrumbs($category, $store): array
    {
        if (!isset($this->categoryCache[$store][$category])) {
            $oldStore = $this->storeManager->getStore();
            $this->storeManager->setCurrentStore($this->storeManager->getStore($store));

            $items = $this->categoryCollectionFactory->create()
                ->addAttributeToFilter('level', ['gteq' => 2])
                ->addNameToResult()
                ->joinUrlRewrite()
                ->setStore($store)
                ->getItems();

            foreach ($items as $item) {
                $this->categoryCache[$store][$item->getId()] = $item;
            }

            $this->storeManager->setCurrentStore($oldStore);
        }

        if (!isset($this->categoryCache[$store][$category])) {
            return [];
        }

        $currentCategory = $this->categoryCache[$store][$category];

        $paths = explode('/', $currentCategory->getPath());

        $result = [];
        foreach ($paths as $path) {
            if (!isset($this->categoryCache[$store][$path])) {
                continue;
            }

            $parentCategory = $this->categoryCache[$store][$path];
            $result[] = [
                'label' => $parentCategory->getName(),
                'url'   => $parentCategory->getRequestPath()
            ];
        }

        return $result;
    }

    /**
     * @param $categoryId
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getIsAnchorCategories($categoryId): array
    {
        if ($this->anchorCategories === null) {
            $categories = $this->categoryCollectionFactory->create()
                ->addAttributeToSelect('is_anchor')
                ->addAttributeToSelect('is_active')
                ->addAttributeToFilter('is_anchor', 1)
                ->addAttributeToFilter('is_active', 1)
                ->getItems();

            foreach ($categories as $category) {
                $childrenIds = $this->getChildren($category, true);

                foreach ($childrenIds as $childId) {
                    $this->anchorCategories[$childId] = $this->getParentIdsByChild($childId);
                }
            }
        }

        if (isset($this->anchorCategories[$categoryId])) {
            return $this->anchorCategories[$categoryId];
        }

        return [];
    }

    /**
     * Return children ids of category
     *
     * @param \Magento\Catalog\Model\Category |\Magento\Catalog\Api\Data\CategoryInterface|\Magento\Catalog\Api\Data\CategoryTreeInterface $magentoCategory
     * @param bool $recursive
     * @param bool $isActive
     * @param bool $sortByPosition
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Exception
     */
    public function getChildren(
        $magentoCategory,
        bool $recursive = false,
        bool $isActive = true,
        bool $sortByPosition = true
    ): array {
        $connection = $this->resourceConnection->getConnection();
        $linkField = $this->metadataPool
            ->getMetadata(CategoryInterface::class)
            ->getLinkField();
        $attributeId = $this->getIsActiveAttributeId();
        $backendTable = $connection->getTableName('catalog_category_entity_int');
        $checkSql = $connection->getCheckSql('c.value_id > 0', 'c.value', 'd.value');

        $bind = [
            'attribute_id' => $attributeId,
            'store_id'     => $magentoCategory->getStoreId(),
            'scope'        => 1,
            'c_path'       => $magentoCategory->getPath() . '/%',
        ];

        $select = $connection->select()->from(
            ['m' => $this->resourceConnection->getTableName('catalog_category_entity')],
            'entity_id'
        )
            ->joinLeft(
                ['d' => $backendTable],
                "d.attribute_id = :attribute_id AND d.store_id = 0 AND d.{$linkField} = m.{$linkField}",
                []
            )->joinLeft(
                ['c' => $backendTable],
                "c.attribute_id = :attribute_id AND c.store_id = :store_id AND c.{$linkField} = m.{$linkField}",
                []
            )
            ->where(
                $connection->quoteIdentifier('path') . ' LIKE :c_path'
            );
        if (!$recursive) {
            $select->where($connection->quoteIdentifier('level') . ' <= :c_level');
            $bind['c_level'] = $magentoCategory->getLevel() + 1;
        }
        if ($isActive) {
            $select->where(
                $checkSql . ' = :scope'
            );
        }
        if ($sortByPosition) {
            $select->order('position ASC');
        }
        $categories = $connection->fetchCol($select, $bind);
        $categoriesIds = [];
        foreach ($categories as $categoryId) {
            $categoriesIds[] = (int)$categoryId;
        }
        return $categoriesIds;
    }

    /**
     * @return int
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getIsActiveAttributeId()
    {
        if ($this->isActiveAttributeId === null) {
            $this->isActiveAttributeId = (int)$this->eavConfig
                ->getAttribute(Category::ENTITY, 'is_active')
                ->getAttributeId();
        }
        return $this->isActiveAttributeId;
    }

    /**
     * @param $categoryId
     * @return array
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getParentIdsByChild($categoryId): array
    {
        /** @var CategoryInterface|Category $category */
        $category = $this->categoryRepository->get($categoryId);

        $output = [];
        foreach ($category->getParentCategories() as $parent) {
            $output[(int)$parent->getEntityId()] = (int)$parent->getPosition();
        }
        return $output;
    }
}
